package org.jboss.portlet.contacts;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.cfg.Configuration;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.MatchMode;
import org.jboss.portlet.JBossActionResponse;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.PortletRequestDispatcher;
import javax.portlet.PortletSecurityException;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.List;

/**
 * User: Chris Mills (millsy@jboss.com)
 * Date: 05-Mar-2006
 * Time: 20:53:43
 */
public class ContactsPortlet extends GenericPortlet
{
   private SessionFactory sessionFactory;

   public void init()
   {
      Configuration config = new Configuration();
      config.configure("WEB-INF/contacts/hibernate.cfg.xml");
      this.sessionFactory = config.buildSessionFactory();
   }

   public void processAction(ActionRequest request, ActionResponse response) throws PortletException, PortletSecurityException, IOException
   {
      String op = request.getParameter("op");
      if((op != null) && (op.trim().length() > 0))
      {
         String key = request.getParameter("key");
         String givenName = request.getParameter("givenName");
         String familyName = request.getParameter("familyName");
         String telephone = request.getParameter("telephone");
         String mobile = request.getParameter("mobile");
         String email = request.getParameter("email");
         if(op.equalsIgnoreCase("Add Contact"))
         {
            addNewContact(request.getUserPrincipal().getName(), givenName, familyName, telephone, mobile, email);
         }
         else if(op.equalsIgnoreCase("Update Contact"))
         {
            saveContact(key, givenName, familyName, telephone, mobile, email);
         }
         else if(op.equalsIgnoreCase("Delete Contact"))
         {
            deleteContact(key);
         }
         else if((request.getParameter("op") != null) && (request.getParameter("op").equalsIgnoreCase("vcard")))
         {
            String vcard = createVCard(request.getParameter("key"));
            ((JBossActionResponse) response).sendStream("text/x-vcard", vcard.getBytes().length, new ByteArrayInputStream(vcard.getBytes()));
         }
      }
   }

   public void doView(RenderRequest request, RenderResponse response)
   {
      try
      {
         if((request.getUserPrincipal() != null) && (request.getUserPrincipal().getName() != null))
         {
            String userName = request.getUserPrincipal().getName();
            String jspName = null;
            if((request.getParameter("op") != null) && (request.getParameter("op").equalsIgnoreCase("Search")))
            {
               request.setAttribute("contacts", getUsersContacts(userName, request.getParameter("familyName")));
               jspName = "/WEB-INF/contacts/list.jsp";
            }
            else if((request.getParameter("op") != null) && (request.getParameter("op").equalsIgnoreCase("shownew")))
            {
               jspName = "/WEB-INF/contacts/add.jsp";
            }
            else if((request.getParameter("op") != null) && (request.getParameter("op").equalsIgnoreCase("edit")))
            {
               request.setAttribute("contact", findContact(userName, request.getParameter("key")));
               jspName = "/WEB-INF/contacts/edit.jsp";
            }
            else if((request.getParameter("op") != null) && (request.getParameter("op").equalsIgnoreCase("delete")))
            {
               request.setAttribute("contact", findContact(userName, request.getParameter("key")));
               jspName = "/WEB-INF/contacts/delete.jsp";
            }
            else
            {
               request.setAttribute("contacts", getUsersContacts(userName));
               jspName = "/WEB-INF/contacts/list.jsp";
            }
            response.setContentType("text/html");
            PortletRequestDispatcher prd = getPortletContext().getRequestDispatcher(jspName);
            prd.include(request, response);
         }
         else
         {
            response.setContentType("text/html");
            PortletRequestDispatcher prd = getPortletContext().getRequestDispatcher("/WEB-INF/common/notloggedin.jsp");
            prd.include(request, response);
         }
      }
      catch(Exception e)
      {
         e.printStackTrace();
      }
   }

   private List getUsersContacts(String userName)
   {
      Session session = this.sessionFactory.openSession();
      Criteria criteria = session.createCriteria(Contact.class);
      criteria.add(Expression.eq("userName", userName));
      List contacts = criteria.list();
      session.disconnect();
      return contacts;
   }

   private List getUsersContacts(String userName, String name)
   {
      Session session = this.sessionFactory.openSession();
      Criteria criteria = session.createCriteria(Contact.class);
      criteria.add(Expression.eq("userName", userName));
      criteria.add(Expression.like("familyName", name, MatchMode.ANYWHERE));
      List contacts = criteria.list();
      session.disconnect();
      return contacts;
   }

   private void addNewContact(String userName, String givenName, String familyName, String telephone, String mobile, String email)
   {
      Session session = this.sessionFactory.openSession();
      Transaction t = session.beginTransaction();
      try
      {
         Contact contact = new Contact(userName);
         contact.setGivenName(givenName);
         contact.setFamilyName(familyName);
         contact.setTelephone(telephone);
         contact.setMobile(mobile);
         contact.setEmail(email);
         session.save(contact);
         t.commit();
      }
      catch(Exception e)
      {
         t.rollback();
      }
      finally
      {
         if(session != null)
         {
            session.disconnect();
         }
      }
   }

   private Contact findContact(String userName, String id)
   {
      Session session = this.sessionFactory.openSession();
      Criteria criteria = session.createCriteria(Contact.class);
      criteria.add(Expression.eq("userName", userName));
      criteria.add(Expression.eq("key", new Long(id)));
      List contacts = criteria.list();
      session.disconnect();
      if(contacts.size() == 1)
      {
         return (Contact) contacts.get(0);
      }
      return null;
   }

   private void saveContact(String key, String givenName, String familyName, String telephone, String mobile, String email)
   {
      Session session = this.sessionFactory.openSession();
      Transaction t = session.beginTransaction();
      try
      {
         Contact contact = (Contact) session.load(Contact.class, new Long(key));
         contact.setGivenName(givenName);
         contact.setFamilyName(familyName);
         contact.setTelephone(telephone);
         contact.setMobile(mobile);
         contact.setEmail(email);

         session.update(contact);
         t.commit();
      }
      catch(Exception e)
      {
         t.rollback();
      }
      finally
      {
         if(session != null)
         {
            session.disconnect();
         }
      }
   }

   private void deleteContact(String key)
   {
      Session session = this.sessionFactory.openSession();
      Transaction t = session.beginTransaction();
      try
      {
         Contact contact = (Contact) session.load(Contact.class, new Long(key));

         session.delete(contact);
         t.commit();
      }
      catch(Exception e)
      {
         t.rollback();
      }
      finally
      {
         if(session != null)
         {
            session.disconnect();
         }
      }
   }

   private String createVCard(String key)
   {
      Session session = this.sessionFactory.openSession();
      Contact contact = (Contact) session.load(Contact.class, new Long(key));
      StringBuffer sbContact = new StringBuffer(1024);
      sbContact.append("BEGIN:VCARD\n");
      sbContact.append("VERSION:2.1\n");
      sbContact.append("N:");
      sbContact.append(contact.getFamilyName());
      sbContact.append(" ");
      sbContact.append(contact.getGivenName());
      sbContact.append("\n");
      sbContact.append("FN:");
      sbContact.append(contact.getGivenName());
      sbContact.append(" ");
      sbContact.append(contact.getFamilyName());
      sbContact.append("\n");
      sbContact.append("TEL;WORK;VOICE:");
      sbContact.append(contact.getTelephone());
      sbContact.append("\n");
      sbContact.append("TEL;CELL;VOICE:");
      sbContact.append(contact.getMobile());
      sbContact.append("\n");
      sbContact.append("EMAIL;PREF;INTERNET:");
      sbContact.append(contact.getEmail());
      sbContact.append("\n");
      sbContact.append("END:VCARD\n");
      return sbContact.toString();
   }
}

